"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var SideralObject_1 = require("./../SideralObject");
var Tool_1 = require("./../Tool");
/**
 * Keyboard event utils
 */
var Keyboard = (function (_super) {
    __extends(Keyboard, _super);
    /* LIFECYCLE */
    /**
     * @constructor
     */
    function Keyboard() {
        var _this = _super.call(this) || this;
        /**
         * list of all inputs key released or pressed relative to the game looping
         * @readonly
         */
        _this.inputs = {};
        /**
         * List of all inputs key released or pressed
         * @private
         */
        _this._inputs = {};
        /**
         * If true, the key event willnot be propaged
         */
        _this.preventInputPropagation = true;
        _this.signals.keyChange = new SideralObject_1.Signal(_this);
        _this.signals.keyPress = new SideralObject_1.Signal(_this);
        _this.signals.keyRelease = new SideralObject_1.Signal(_this);
        _this.connect(_this.signals.update, _this._updateInputs.bind(_this));
        return _this;
    }
    /**
     * @override
     */
    Keyboard.prototype.initialize = function (props) {
        if (props === void 0) { props = {}; }
        _super.prototype.initialize.call(this, props);
        window.addEventListener("keydown", this._onKeydown.bind(this));
        window.addEventListener("keyup", this._onKeyup.bind(this));
    };
    /**
     * @override
     */
    Keyboard.prototype.kill = function () {
        _super.prototype.kill.call(this);
        window.removeEventListener("keydown", this._onKeydown.bind(this));
        window.removeEventListener("keyup", this._onKeydown.bind(this));
    };
    /* EVENTS */
    /**
     * Update all device inputs
     * @private
     */
    Keyboard.prototype._updateInputs = function () {
        for (var key in this._inputs) {
            if (!this._inputs.hasOwnProperty(key)) {
                continue;
            }
            var input = this.inputs[key], _input = this._inputs[key];
            // Pressed
            if (_input === Tool_1.Enum.KEY_STATE.PRESSED) {
                if (input === _input) {
                    this.inputs[key] = Tool_1.Enum.KEY_STATE.HOLD;
                }
                else if (input !== Tool_1.Enum.KEY_STATE.HOLD) {
                    this.inputs[key] = Tool_1.Enum.KEY_STATE.PRESSED;
                    this.signals.keyChange.dispatch(key, true);
                    this.signals.keyPress.dispatch(key);
                }
                // Released
            }
            else if (_input === Tool_1.Enum.KEY_STATE.RELEASED) {
                if (!input) {
                    this.inputs[key] = Tool_1.Enum.KEY_STATE.PRESSED;
                }
                else if (input === _input) {
                    delete this.inputs[key];
                    delete this._inputs[key];
                }
                else {
                    this.inputs[key] = Tool_1.Enum.KEY_STATE.RELEASED;
                    this.signals.keyChange.dispatch(key, false);
                    this.signals.keyRelease.dispatch(key);
                }
            }
        }
    };
    /**
     * Event on keydown
     * @param e - The event
     * @returns Input propagation
     */
    Keyboard.prototype._onKeydown = function (e) {
        if (this.preventInputPropagation) {
            e.preventDefault();
            e.stopPropagation();
        }
        this._inputs[e.keyCode] = Tool_1.Enum.KEY_STATE.PRESSED;
        return !this.preventInputPropagation;
    };
    /**
     * Event on keyup
     * @param e - The event
     * @returns Input propagation
     */
    Keyboard.prototype._onKeyup = function (e) {
        if (this.preventInputPropagation) {
            e.preventDefault();
            e.stopPropagation();
        }
        this._inputs[e.keyCode] = Tool_1.Enum.KEY_STATE.RELEASED;
        return !this.preventInputPropagation;
    };
    return Keyboard;
}(SideralObject_1.SideralObject));
exports.Keyboard = Keyboard;
